@@ -1,6 +1,6 @@
 ;;;; libbrlapi - A library providing access to braille terminals for applications.
 ;;;;
-;;;; Copyright (C) 2006-2024 by Mario Lang <mlang@delysid.org>
+;;;; Copyright (C) 2006-2025 by Mario Lang <mlang@tugraz.at>
 ;;;;
 ;;;; libbrlapi comes with ABSOLUTELY NO WARRANTY.
 ;;;;
@@ -103,7 +103,7 @@ The sublist for each returned property contains, in order, its name, its value,
   (host :string))
 
 (defun open-connection (&key auth host)
-  "Open a new connection to BRLTTY on HOST usng AUTH for authorization.
+  "Open a new connection to BRLTTY on HOST using AUTH for authorization.
 Return a DISPLAY object which can further be used to interact with BRLTTY."
   (with-foreign-object (settings 'settings)
     (setf (foreign-slot-value settings 'settings 'auth)
