@@ -1,7 +1,7 @@
 /*
  * libbrlapi - A library providing access to braille terminals for applications.
  *
- * Copyright (C) 2006-2024 by Dave Mielke <dave@mielke.cc>
+ * Copyright (C) 2006-2025 by Dave Mielke <dave@mielke.cc>
  *
  * libbrlapi comes with ABSOLUTELY NO WARRANTY.
  *
@@ -41,7 +41,7 @@ static int openConnection(lua_State *L) {
   );
   luaL_setmetatable(L, handle_t);
 
-  if (brlapi__openConnection(handle, &desiredSettings, &actualSettings) == -1)
+  if (brlapi__openConnection(handle, &desiredSettings, &actualSettings) == BRLAPI_INVALID_FILE_DESCRIPTOR)
     error(L);
 
   lua_pushstring(L, actualSettings.host);
