@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2019 by The BRLTTY Developers.
+ * Copyright (C) 1995-2025 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -17,6 +17,7 @@
  */
 
 #include "prologue.h"
+#include "embed.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -27,7 +28,7 @@
 
 #ifdef HAVE_ATSPI_GET_A11Y_BUS
 #include <atspi/atspi.h>
-#endif
+#endif /* HAVE_ATSPI_GET_A11Y_BUS */
 
 #ifdef __MINGW32__
 #include "win_pthread.h"
@@ -56,14 +57,21 @@
 #include <X11/keysym.h>
 #endif /* HAVE_X11_KEYSYM_H */
 
+#ifdef HAVE_PKG_X11
+#include "xsel.h"
+#include "clipboard.h"
+#endif /* HAVE_PKG_X11 */
+
 #include "log.h"
+#include "report.h"
 #include "parse.h"
 #include "thread.h"
 #include "brl_cmds.h"
-#include "charset.h"
+#include "async_handle.h"
 #include "async_io.h"
 #include "async_alarm.h"
 #include "async_event.h"
+#include "unicode.h"
 
 typedef enum {
   PARM_RELEASE,
@@ -74,9 +82,9 @@ typedef enum {
 #include "scr_driver.h"
 
 typedef enum {
-  TYPE_TEXT,
-  TYPE_TERMINAL,
   TYPE_ALL,
+  TYPE_TERMINAL,
+  TYPE_TEXT,
   TYPE_COUNT
 } TypeValue;
 
@@ -85,17 +93,27 @@ static unsigned char typeFlags[TYPE_COUNT];
 
 static char *curSender;
 static char *curPath;
+static char *curRole;
+static char *curName; /* If not a text widget, at least record some name */
+static ScreenContentQuality curQuality;
 
 static long curNumRows, curNumCols;
 static wchar_t **curRows;
 static long *curRowLengths;
-static long curCaret,curPosX,curPosY;
+static long curCaret; /* In unicode codepoints */
+static long curPosX,curPosY; /* In screen offset */
 
 static DBusConnection *bus = NULL;
 
 static int updated;
 
-/* having our own implementation is much more independant on locales */
+#ifdef HAVE_PKG_X11
+static Display *dpy;
+static XSelData xselData;
+static char *clipboardContent;
+#endif /* HAVE_PKG_X11 */
+
+/* having our own implementation is much more independent on locales */
 
 typedef struct {
   int remaining;
@@ -189,9 +207,15 @@ static size_t my_mbsrtowcs(wchar_t *dest, const char **src, size_t len, my_mbsta
   while (len-put || !dest) {
     skip = my_mbrtowc(bufp, *src, 6, ps);
     switch (skip) {
-      case (size_t)(-2): errno = EILSEQ; /* shouldn't happen ! */
-      case (size_t)(-1): return (size_t)(-1);
-      case 0: *src = NULL; return put;
+      case (size_t)(-2):
+        errno = EILSEQ; /* shouldn't happen ! */
+        /* fall through */
+      case (size_t)(-1):
+        return (size_t)(-1);
+
+      case 0:
+        *src = NULL;
+        return put;
     }
     *src += skip;
     if (dest) bufp++;
@@ -252,46 +276,43 @@ processParameters_AtSpi2Screen (char **parameters) {
     }
   }
 
-  typeFlags[TYPE_TEXT] = 0;
-  typeFlags[TYPE_TERMINAL] = 1;
-  typeFlags[TYPE_ALL] = 0;
   {
     const char *parameter = parameters[PARM_TYPE];
 
+    for (unsigned int index=0; index<TYPE_COUNT; index+=1) {
+      typeFlags[index] = 0;
+    }
+
     if (*parameter) {
-      int count;
-      char **types = splitString(parameter, '+', &count);
+      if (!isAbbreviation("default", parameter)) {
+        int count;
+        char **types = splitString(parameter, '+', &count);
 
-      if (types) {
-        static const char *const choices[] = {
-          [TYPE_TEXT] = "text",
-          [TYPE_TERMINAL] = "terminal",
-          [TYPE_ALL] = "all",
-          [TYPE_COUNT] = NULL
-        };
+        if (types) {
+          static const char *const choices[] = {
+            [TYPE_ALL] = "all",
+            [TYPE_TERMINAL] = "terminal",
+            [TYPE_TEXT] = "text",
+            [TYPE_COUNT] = NULL
+          };
 
-        for (unsigned int index=0; index<TYPE_COUNT; index+=1) {
-          typeFlags[index] = 0;
-        }
+          for (unsigned int index=0; index<count; index+=1) {
+            const char *type = types[index];
+            unsigned int choice;
 
-        for (unsigned int index=0; index<count; index+=1) {
-          const char *type = types[index];
-          unsigned int choice;
-
-          if (validateChoice(&choice, type, choices)) {
-            if ((choice == TYPE_ALL) && (index > 0)) {
+            if (!validateChoice(&choice, type, choices)) {
+              logMessage(LOG_WARNING, "%s: %s", "invalid widget type", type);
+            } else if ((choice == TYPE_ALL) && (index > 0)) {
               logMessage(LOG_WARNING, "widget type is mutually exclusive: %s", type);
             } else if (typeFlags[choice] || typeFlags[TYPE_ALL]) {
               logMessage(LOG_WARNING, "widget type specified more than once: %s", type);
             } else {
               typeFlags[choice] = 1;
             }
-          } else {
-            logMessage(LOG_WARNING, "%s: %s", "invalid widget type", type);
           }
-        }
 
-        deallocateStrings(types);
+          deallocateStrings(types);
+        }
       }
     }
   }
@@ -353,9 +374,9 @@ send_with_reply_and_block(DBusConnection *bus, DBusMessage *msg, int timeout_ms,
   return reply;
 }
 
-static void findPosition(long position, long *px, long *py) {
-  long offset=0, newoffset, x, y;
-  /* XXX: I don't know what they do with necessary combining accents */
+/* px, py are counting unicode codepoints, psx is counting screen coordinate */
+static void findPosition(long position, long *px, long *py, long *psx) {
+  long offset=0, newoffset, x, y, sx;
   for (y=0; y<curNumRows; y++) {
     if ((newoffset = offset + curRowLengths[y]) > position)
       break;
@@ -374,28 +395,108 @@ static void findPosition(long position, long *px, long *py) {
     }
   } else
     x = position-offset;
-  *px = x;
+
+  /* Process text of the line until the wanted position */
+  for (offset=0, sx=0; offset<x && offset<curRowLengths[y]; offset++) {
+    wchar_t wc = curRows[y][offset];
+
+    if (wc == '\n') continue; /* EOL, ignore */
+
+    if (wc == '\t') {
+      /* Switch to next tab */
+      sx = (sx+1+7)&~7;
+      continue;
+    }
+
+    if (getCharacterWidth(wc) != 0) sx++;
+  }
+
+  if (px) *px = x;
   *py = y;
+  if (psx) *psx = sx;
+}
+
+/* Convert from screen coordinates to unicode codepoints offset */
+static long findCoordinates(long xx, long yy) {
+  long offset=0, x, y, sx;
+  if (yy >= curNumRows) {
+    return -1;
+  }
+  for (y=0; y<yy; y++) {
+    offset += curRowLengths[y];
+  }
+
+  /* Process text of the line until the wanted position */
+  for (x=0, sx=0; sx<xx && x<curRowLengths[y]; x++) {
+    wchar_t wc = curRows[y][x];
+
+    if (wc == '\n') continue; /* EOL, ignore */
+
+    if (wc == '\t') {
+      /* Switch to next tab */
+      sx = (sx+1+7)&~7;
+      continue;
+    }
+
+    if (getCharacterWidth(wc) != 0) sx++;
+  }
+
+  if (x >= curRowLengths[y])
+    x = curRowLengths[y]-1;
+  return offset+x;
 }
 
 static void caretPosition(long caret) {
-  findPosition(caret,&curPosX,&curPosY);
+  if (caret < 0) {
+    caret = 0;
+  }
+  findPosition(caret,NULL,&curPosY,&curPosX);
   curCaret = caret;
 }
 
 static void finiTerm(void) {
+  unsigned i;
   logMessage(LOG_CATEGORY(SCREEN_DRIVER),
              "end of term %s:%s",curSender,curPath);
   free(curSender);
   curSender = NULL;
   free(curPath);
   curPath = NULL;
+  free(curRole);
+  curRole = NULL;
+  free(curName);
+  curName = NULL;
   curPosX = curPosY = 0;
-  free(curRows);
+  if (curRows) {
+    for (i=0;i<curNumRows;i++)
+      free(curRows[i]);
+    free(curRows);
+  }
   curRows = NULL;
+  free(curRowLengths);
+  curRowLengths = NULL;
   curNumCols = curNumRows = 0;
 }
 
+#define ROLE_TERMINAL "terminal"
+#define ROLE_TEXT "text"
+
+static int
+isRole (const char *role) {
+  if (!curRole) return 0;
+  return strcmp(curRole, role) == 0;
+}
+
+static int
+isTerminal (void) {
+  return isRole(ROLE_TERMINAL);
+}
+
+static int
+isText (void) {
+  return isRole(ROLE_TEXT);
+}
+
 /* Get the role of an AT-SPI2 object */
 static char *getRole(const char *sender, const char *path) {
   const char *text;
@@ -424,6 +525,76 @@ out:
   return res;
 }
 
+/* Get the get interfaces of an AT-SPI2 object */
+static int getHasTextInterface(const char *sender, const char *path) {
+  DBusMessage *msg, *reply;
+  DBusMessageIter iter;
+  DBusMessageIter iter_array;
+  int ret = 0;
+
+  msg = new_method_call(sender, path, SPI2_DBUS_INTERFACE_ACCESSIBLE, "GetInterfaces");
+  if (!msg)
+    return 0;
+  reply = send_with_reply_and_block(bus, msg, 1000, "getting interfaces");
+  if (!reply)
+    return 0;
+
+  dbus_message_iter_init(reply, &iter);
+  dbus_message_iter_recurse (&iter, &iter_array);
+  while (dbus_message_iter_get_arg_type (&iter_array) != DBUS_TYPE_INVALID)
+  {
+    const char *iface;
+    dbus_message_iter_get_basic (&iter_array, &iface);
+
+    if (!strcmp (iface, "org.a11y.atspi.Text"))
+    {
+      ret = 1;
+      goto out;
+    }
+    dbus_message_iter_next (&iter_array);
+  }
+
+out:
+  dbus_message_unref(reply);
+  return ret;
+}
+
+static char *getName(const char *sender, const char *path) {
+  const char *name;
+  char *res = NULL;
+  DBusMessage *msg, *reply = NULL;
+  const char *interface = SPI2_DBUS_INTERFACE_ACCESSIBLE;
+  const char *property = "Name";
+  DBusMessageIter iter, iter_variant;
+
+  msg = new_method_call(sender, path, DBUS_INTERFACE_PROPERTIES, "Get");
+  if (!msg)
+    return NULL;
+  dbus_message_append_args(msg, DBUS_TYPE_STRING, &interface, DBUS_TYPE_STRING, &property, DBUS_TYPE_INVALID);
+  reply = send_with_reply_and_block(bus, msg, 1000, "getting name");
+  if (!reply)
+    return NULL;
+
+  dbus_message_iter_init(reply, &iter);
+  if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_VARIANT) {
+    logMessage(LOG_CATEGORY(SCREEN_DRIVER),
+               "getName didn't return a variant but '%c'", dbus_message_iter_get_arg_type(&iter));
+    goto out;
+  }
+  dbus_message_iter_recurse(&iter, &iter_variant);
+  if (dbus_message_iter_get_arg_type(&iter_variant) != DBUS_TYPE_STRING) {
+    logMessage(LOG_CATEGORY(SCREEN_DRIVER),
+               "getName didn't return a variant but '%c'", dbus_message_iter_get_arg_type(&iter_variant));
+    goto out;
+  }
+  dbus_message_iter_get_basic(&iter_variant, &name);
+  res = strdup(name);
+
+out:
+  dbus_message_unref(reply);
+  return res;
+}
+
 /* Get the text of an AT-SPI2 object */
 static char *getText(const char *sender, const char *path) {
   const char *text;
@@ -474,13 +645,13 @@ static dbus_int32_t getCaret(const char *sender, const char *path) {
   dbus_message_iter_init(reply, &iter);
   if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_VARIANT) {
     logMessage(LOG_CATEGORY(SCREEN_DRIVER),
-               "getText didn't return a variant but '%c'", dbus_message_iter_get_arg_type(&iter));
+               "getCaret didn't return a variant but '%c'", dbus_message_iter_get_arg_type(&iter));
     goto out;
   }
   dbus_message_iter_recurse(&iter, &iter_variant);
   if (dbus_message_iter_get_arg_type(&iter_variant) != DBUS_TYPE_INT32) {
     logMessage(LOG_CATEGORY(SCREEN_DRIVER),
-               "getText didn't return an int32 but '%c'", dbus_message_iter_get_arg_type(&iter_variant));
+               "getCaret didn't return an int32 but '%c'", dbus_message_iter_get_arg_type(&iter_variant));
     goto out;
   }
   dbus_message_iter_get_basic(&iter_variant, &res);
@@ -494,22 +665,20 @@ out:
 
 /* Switched to a new terminal, restart from scratch */
 static void restartTerm(const char *sender, const char *path) {
+  char *text = getText(sender, path);
+  if (!text) {
+    text = getName(sender, path);
+    if (!text) return;
+  }
+
   char *c,*d;
   const char *e;
   long i,len;
-  char *text;
-
-  if (curPath)
-    finiTerm();
-
-  text = getText(sender, path);
-  if (!text)
-    return;
 
   curSender = strdup(sender);
   curPath = strdup(path);
   logMessage(LOG_CATEGORY(SCREEN_DRIVER),
-             "new term %s:%s with text %s",curSender,curPath, text);
+             "new term %s:%s with text %s", curSender, curPath, text);
 
   if (curRows) {
     for (i=0;i<curNumRows;i++)
@@ -544,7 +713,7 @@ static void restartTerm(const char *sender, const char *path) {
 	logMessage(LOG_ERR,"unterminated sequence %s",c);
       else if (len==-1)
 	logSystemError("mbrlen");
-      curRowLengths[i] = (len = -1) + (d != NULL);
+      curRowLengths[i] = (len = 0) + (d != NULL);
     }
     curRows[i] = malloc((len + (d!=NULL)) * sizeof(*curRows[i]));
     e = c;
@@ -563,18 +732,52 @@ static void restartTerm(const char *sender, const char *path) {
 
 /* Switched to a new object, check whether we want to read it, and if so, restart with it */
 static void tryRestartTerm(const char *sender, const char *path) {
-  char *role = getRole(sender, path);
+  if (curName) {
+    free(curName);
+    curName = NULL;
+  }
+  if (curPath) finiTerm();
+
+  curRole = getRole(sender, path);
   logMessage(LOG_CATEGORY(SCREEN_DRIVER),
-             "state changed focus to role %s", role);
-  if (typeFlags[TYPE_ALL] ||
-      (role && typeFlags[TYPE_TEXT] && (strcmp(role, "text") == 0)) ||
-      (role && typeFlags[TYPE_TERMINAL] && (strcmp(role, "terminal") == 0))) {
+             "state changed focus to role %s", curRole);
+
+  curQuality = getHasTextInterface(sender, path)? SCQ_POOR: SCQ_LOW;
+  unsigned char requested = typeFlags[TYPE_ALL];
+
+  if (!requested) {
+    if (isTerminal()) {
+      curQuality = SCQ_GOOD;
+      requested = typeFlags[TYPE_TERMINAL];
+    } else if (isText()) {
+      curQuality = SCQ_FAIR;
+      requested = typeFlags[TYPE_TEXT];
+    }
+  }
+
+  if (requested) curQuality = SCQ_GOOD;     
+
+  if (curQuality != SCQ_LOW) {
     restartTerm(sender, path);
   } else {
-    if (curPath)
-      finiTerm();
+    curName = getName(sender, path);
+    if (curName && !curName[0]) {
+      free(curName);
+      curName = NULL;
+    }
+    if (curName)
+      logMessage(LOG_CATEGORY(SCREEN_DRIVER), "got name %s", curName);
+    else {
+      curName = getRole(sender, path);
+      if (curName)
+	logMessage(LOG_CATEGORY(SCREEN_DRIVER), "got role %s", curName);
+      else {
+	logMessage(LOG_CATEGORY(SCREEN_DRIVER), "did not get name or role");
+	curName = strdup("unknown widget");
+	curQuality = SCQ_NONE;
+      }
+    }
   }
-  free(role);
 }
 
 /* Get the state of an object */
@@ -608,7 +811,8 @@ static dbus_uint32_t *getState(const char *sender, const char *path)
     goto out;
   }
   ret = malloc(sizeof(*ret) * count);
-  memcpy(ret, states, sizeof(*ret) * count);
+  if (ret)
+    memcpy(ret, states, sizeof(*ret) * count);
 
 out:
   dbus_message_unref(reply);
@@ -870,7 +1074,15 @@ static void AtSpi2HandleEvent(const char *interface, DBusMessage *message)
     if (!curSender || strcmp(sender, curSender) || strcmp(path, curPath)) return;
     logMessage(LOG_CATEGORY(SCREEN_DRIVER),
                "delete %d from %d",detail2,detail1);
-    findPosition(detail1,&x,&y);
+    if (detail1 < 0) {
+      logMessage(LOG_ERR,"deleting %ld %d before beginning of text!", toDelete, detail1);
+      toDelete -= -detail1;
+      detail1 = 0;
+    }
+    if (toDelete <= 0) {
+      return;
+    }
+    findPosition(detail1,&x,&y,NULL);
     if (dbus_message_iter_get_arg_type(&iter_variant) != DBUS_TYPE_STRING) {
       logMessage(LOG_CATEGORY(SCREEN_DRIVER),
                  "ergl, not string but '%c'", dbus_message_iter_get_arg_type(&iter_variant));
@@ -889,13 +1101,17 @@ static void AtSpi2HandleEvent(const char *interface, DBusMessage *message)
       else {
 	/* imaginary extra line doesn't provide more length, and shouldn't need to ! */
 	if (x+toDelete > length) {
-	  logMessage(LOG_ERR,"deleting past end of text !");
+	  logMessage(LOG_ERR,"deleting %ld %ld past end of text !", toDelete, x+toDelete - length);
 	  /* discarding */
+	  if (x > length)
+	    x = length;
 	  toDelete = length - x;
 	}
 	break; /* deleting up to end */
       }
     }
+    if (toDelete <= 0)
+      return;
     if (length-toDelete>0) {
       /* still something on line y */
       if (y!=downTo) {
@@ -915,7 +1131,9 @@ static void AtSpi2HandleEvent(const char *interface, DBusMessage *message)
     if (downTo>=curNumRows)
       /* imaginary extra lines don't need to be deleted */
       downTo=curNumRows-1;
-    delRows(y+1,downTo-y);
+    if (downTo>y) {
+      delRows(y+1,downTo-y);
+    }
     caretPosition(curCaret);
   } else if (!strcmp(interface, "Object") && !strcmp(member, "TextChanged") && !strcmp(detail, "insert")) {
     long len=detail2,semilen,x,y;
@@ -924,16 +1142,24 @@ static void AtSpi2HandleEvent(const char *interface, DBusMessage *message)
     if (!curSender || strcmp(sender, curSender) || strcmp(path, curPath)) return;
     logMessage(LOG_CATEGORY(SCREEN_DRIVER),
                "insert %d from %d",detail2,detail1);
-    findPosition(detail1,&x,&y);
+    findPosition(detail1,&x,&y,NULL);
     if (dbus_message_iter_get_arg_type(&iter_variant) != DBUS_TYPE_STRING) {
       logMessage(LOG_CATEGORY(SCREEN_DRIVER),
                  "ergl, not string but '%c'", dbus_message_iter_get_arg_type(&iter_variant));
       return;
     }
+    if (detail1 < 0) {
+      logMessage(LOG_ERR,"adding %ld %d before beginning of text!", len, detail1);
+      detail1 = 0;
+    }
     dbus_message_iter_get_basic(&iter_variant, &added);
     logMessage(LOG_CATEGORY(SCREEN_DRIVER),
                "'%s'",added);
     adding = c = added;
+    if (y < curNumRows && x > curRowLengths[y]) {
+      logMessage(LOG_ERR,"adding %ld %ld past end of text!", len, x - curRowLengths[y]);
+      x = curRowLengths[y];
+    }
     if (x && (c = strchr(adding,'\n'))) {
       /* splitting line */
       addRows(y,1);
@@ -990,38 +1216,103 @@ static void AtSpi2HandleEvent(const char *interface, DBusMessage *message)
   updated = 1;
 }
 
-static DBusHandlerResult AtSpi2Filter(DBusConnection *connection, DBusMessage *message, void *user_data)
-{
-  int type = dbus_message_get_type(message);
-  const char *interface = dbus_message_get_interface(message);
-  const char *member = dbus_message_get_member(message);
+static int closeX = 1;
 
-  if (type == DBUS_MESSAGE_TYPE_SIGNAL) {
-    if (!strncmp(interface, SPI2_DBUS_INTERFACE_EVENT".", strlen(SPI2_DBUS_INTERFACE_EVENT"."))) {
-      AtSpi2HandleEvent(interface + strlen(SPI2_DBUS_INTERFACE_EVENT"."), message);
-    } else if (!strcmp(interface, DBUS_INTERFACE_LOCAL) &&
-               !strcmp(member, "Disconnected")) {
-      logMessage(LOG_CATEGORY(SCREEN_DRIVER),
-                 "DBus disconnected signal, shutting down");
-#if defined(SIGTERM)
-      raise(SIGTERM);
-#elif defined(SIGINT)
-      raise(SIGINT);
-#else /* termination signal */
-#warning no defined termination signal
-#endif /* termination signal */
-    } else {
-      logMessage(LOG_CATEGORY(SCREEN_DRIVER),
-                 "unknown signal: Intf:%s Msg:%s", interface, member);
-    }
-  } else {
-    logMessage(LOG_CATEGORY(SCREEN_DRIVER),
-               "unknown message: Type:%d Intf:%s Msg:%s", type, interface, member);
+static void
+onScreenDriverFailure (const char *cause) {
+  logMessage(LOG_ERR, "screen driver failure: %s", cause);
+  closeX = 1;
+  brlttyInterrupt(WAIT_STOP);
+}
+
+#ifdef HAVE_PKG_X11
+/* Integration of X11 events with brltty monitors */
+static AsyncHandle a2XWatch;
+static ReportListenerInstance *coreSelUpdatedListener;
+static int settingClipboard;
+
+/* Called when X selection got updated, update the BRLTTY clipboard content */
+void a2XSelUpdated(const char *data, unsigned long size) {
+  if (!data) return;
+
+  char content[size + 1];
+  memcpy(content, data, size);
+  content[size] = 0;
+
+  logMessage(LOG_CATEGORY(SCREEN_DRIVER), "X Selection got '%s'", content);
+  settingClipboard = 1;
+  setMainClipboardContent(content);
+  settingClipboard = 0;
+}
+
+#ifdef HAVE_PTHREAD_ATFORK
+/* Drop X connection without shutting down its resources. Needed at fork. */
+void a2DropX(void) {
+  close(XConnectionNumber(dpy));
+  dpy = NULL;
+}
+#endif /* HAVE_PTHREAD_ATFORK */
+
+/* Called when X events are available, process them */
+ASYNC_MONITOR_CALLBACK(a2ProcessX) {
+  XEvent ev;
+
+  if (closeX) {
+    asyncCancelRequest(a2XWatch);
+    a2XWatch = NULL;
+    return 1;
   }
 
-  return DBUS_HANDLER_RESULT_HANDLED;
+  while (XPending(dpy)) {
+    XNextEvent(dpy, &ev);
+    XSelProcess(dpy, &xselData, &ev, clipboardContent, a2XSelUpdated);
+  }
+
+  return 1;
 }
 
+/* Called when BRLTTY selection got updated, update the X clipboard content */
+REPORT_LISTENER(a2CoreSelUpdated) {
+  const ApiParameterUpdatedReport *report = parameters->reportData;
+  if (report->parameter != BRLAPI_PARAM_CLIPBOARD_CONTENT) return;
+  if (settingClipboard) return;
+  char *newContent = getMainClipboardContent();
+
+  if (newContent) {
+    if (!clipboardContent || (strcmp(clipboardContent, newContent) != 0)) {
+      free(clipboardContent);
+      logMessage(LOG_CATEGORY(SCREEN_DRIVER), "core Selection got '%s'", newContent);
+      clipboardContent = newContent;
+      XSelSet(dpy, &xselData);
+    } else {
+      free(newContent);
+    }
+  }
+}
+
+static int ErrorHandler(Display *dpy, XErrorEvent *ev) {
+  char buffer[128];
+  XGetErrorText(dpy, ev->error_code, buffer, sizeof(buffer));
+  logMessage(LOG_ERR, "X Error %d, %s", ev->type, buffer);
+  logMessage(LOG_ERR, "resource %#010lx, req %u:%u",ev->resourceid,ev->request_code,ev->minor_code);
+  onScreenDriverFailure("X error");
+  return 0;
+}
+
+static int IOErrorHandler(Display *dpy) {
+  onScreenDriverFailure("X I/O error");
+  return 0;
+}
+
+#ifdef HAVE_XSETIOERROREXITHANDLER
+static void IOErrorExitHandler(Display *dpy, void *data) {
+  onScreenDriverFailure("X I/O Error Exit");
+  return;
+}
+#endif
+
+#endif /* HAVE_PKG_X11 */
+
 /* Integration of DBus watches with brltty monitors */
 
 struct a2Watch
@@ -1233,12 +1524,39 @@ addWatches (void) {
   };
 
   for (const WatchEntry *watch=watchTable; watch->message; watch+=1) {
-    if (!addWatch(watch->message, watch->event)) return 0;
+    if (!addWatch(watch->message, watch->event)) {
+      logMessage(LOG_ERR, "can't add watch %s %s", watch->message, watch->event);
+      return 0;
+    }
   }
 
   return 1;
 }
 
+static DBusHandlerResult AtSpi2Filter(DBusConnection *connection, DBusMessage *message, void *user_data)
+{
+  int type = dbus_message_get_type(message);
+  const char *interface = dbus_message_get_interface(message);
+  const char *member = dbus_message_get_member(message);
+
+  if (type == DBUS_MESSAGE_TYPE_SIGNAL) {
+    if (!strncmp(interface, SPI2_DBUS_INTERFACE_EVENT".", strlen(SPI2_DBUS_INTERFACE_EVENT"."))) {
+      AtSpi2HandleEvent(interface + strlen(SPI2_DBUS_INTERFACE_EVENT"."), message);
+    } else if (!strcmp(interface, DBUS_INTERFACE_LOCAL) &&
+               !strcmp(member, "Disconnected")) {
+      onScreenDriverFailure("DBus disconnected");
+    } else {
+      logMessage(LOG_CATEGORY(SCREEN_DRIVER),
+                 "unknown signal: Intf:%s Msg:%s", interface, member);
+    }
+  } else {
+    logMessage(LOG_CATEGORY(SCREEN_DRIVER),
+               "unknown message: Type:%d Intf:%s Msg:%s", type, interface, member);
+  }
+
+  return DBUS_HANDLER_RESULT_HANDLED;
+}
+
 static int
 construct_AtSpi2Screen (void) {
   DBusError error;
@@ -1247,7 +1565,7 @@ construct_AtSpi2Screen (void) {
 #ifdef HAVE_ATSPI_GET_A11Y_BUS
   bus = atspi_get_a11y_bus();
   if (!bus)
-#endif
+#endif /* HAVE_ATSPI_GET_A11Y_BUS */
   {
     bus = dbus_bus_get(DBUS_BUS_SESSION, &error);
     if (dbus_error_is_set(&error)) {
@@ -1262,7 +1580,10 @@ construct_AtSpi2Screen (void) {
     goto noBus;
   }
 
-  if (!dbus_connection_add_filter(bus, AtSpi2Filter, NULL, NULL)) goto noConnection;
+  if (!dbus_connection_add_filter(bus, AtSpi2Filter, NULL, NULL)) {
+    logMessage(LOG_ERR, "can't add atspi2 filter");
+    goto noConnection;
+  }
   if (!addWatches()) goto noWatches;
 
   if (!curPath) {
@@ -1276,7 +1597,27 @@ construct_AtSpi2Screen (void) {
   dbus_connection_set_watch_functions(bus, a2AddWatch, a2RemoveWatch, a2WatchToggled, NULL, NULL);
   dbus_connection_set_timeout_functions(bus, a2AddTimeout, a2RemoveTimeout, a2TimeoutToggled, NULL, NULL);
 
+#ifdef HAVE_PKG_X11
+  closeX = 0;
+  dpy = XOpenDisplay(NULL);
+  if (dpy) {
+    XSetErrorHandler(ErrorHandler);
+    XSetIOErrorHandler(IOErrorHandler);
+#ifdef HAVE_XSETIOERROREXITHANDLER
+    XSetIOErrorExitHandler(dpy, IOErrorExitHandler, NULL);
+#endif
+    XSelInit(dpy, &xselData);
+    XFlush(dpy);
+#ifdef HAVE_PTHREAD_ATFORK
+    pthread_atfork(NULL, NULL, a2DropX);
+#endif /* HAVE_PTHREAD_ATFORK */
+    asyncMonitorFileInput(&a2XWatch, XConnectionNumber(dpy), a2ProcessX, NULL);
+    coreSelUpdatedListener = registerReportListener(REPORT_API_PARAMETER_UPDATED, a2CoreSelUpdated , NULL);
+  }
+#endif /* HAVE_PKG_X11 */
+
   logMessage(LOG_CATEGORY(SCREEN_DRIVER), "SPI2 initialized");
+  brlttyEnableInterrupt();
   return 1;
 
 noWatches:
@@ -1285,24 +1626,43 @@ noConnection:
   dbus_connection_unref(bus);
 
 noBus:
+  onScreenDriverFailure("driver couldn't start");
   return 0;
 }
 
 static void
 destruct_AtSpi2Screen (void) {
+  brlttyDisableInterrupt();
+#ifdef HAVE_PKG_X11
+  if (dpy) {
+    if (coreSelUpdatedListener) {
+      unregisterReportListener(coreSelUpdatedListener);
+      coreSelUpdatedListener = NULL;
+    }
+    if (a2XWatch) {
+      asyncCancelRequest(a2XWatch);
+      a2XWatch = NULL;
+    }
+    XCloseDisplay(dpy);
+    dpy = NULL;
+    free(clipboardContent);
+    clipboardContent = NULL;
+  }
+#endif /* HAVE_PKG_X11 */
   dbus_connection_remove_filter(bus, AtSpi2Filter, NULL);
   dbus_connection_close(bus);
   dbus_connection_unref(bus);
   logMessage(LOG_CATEGORY(SCREEN_DRIVER),
              "SPI2 stopped");
+  finiTerm();
 }
 
 static int
 currentVirtualTerminal_AtSpi2Screen (void) {
-  return (curPath || !releaseScreen)? 0: SCR_NO_VT;
+  return (curPath || curName || !releaseScreen)? 0: SCR_NO_VT;
 }
 
-static const char nonatspi [] = "not an AT-SPI2 text widget";
+static const char msgNotAtSpi [] = "not an AT-SPI2 text widget";
 
 static int
 poll_AtSpi2Screen (void)
@@ -1327,14 +1687,22 @@ describe_AtSpi2Screen (ScreenDescription *description) {
     description->rows = curNumRows?curNumRows:1;
     description->posx = curPosX;
     description->posy = curPosY;
+    description->quality = curQuality;
+  } else if (curName) {
+    description->rows = 1;
+    description->cols = strlen(curName) + 2;
+    description->posx = 0;
+    description->posy = 0;
+    description->quality = curQuality;
   } else {
-    const char *message = nonatspi;
+    const char *message = msgNotAtSpi;
     if (releaseScreen) description->unreadable = message;
 
     description->rows = 1;
     description->cols = strlen(message);
     description->posx = 0;
     description->posy = 0;
+    description->quality = SCQ_NONE;
   }
 
   description->number = currentVirtualTerminal_AtSpi2Screen();
@@ -1345,7 +1713,13 @@ readCharacters_AtSpi2Screen (const ScreenBox *box, ScreenCharacter *buffer) {
   clearScreenCharacters(buffer, (box->height * box->width));
 
   if (!curPath) {
-    setScreenMessage(box, buffer, nonatspi);
+    if (curName) {
+      char msg[strlen(curName)+3];
+      snprintf(msg, sizeof(msg), "[%s]", curName);
+      setScreenMessage(box, buffer, msg);
+    }
+    else
+      setScreenMessage(box, buffer, msgNotAtSpi);
     return 1;
   }
 
@@ -1354,11 +1728,34 @@ readCharacters_AtSpi2Screen (const ScreenBox *box, ScreenCharacter *buffer) {
   if (!validateScreenBox(box, cols, curNumRows)) return 0;
 
   for (unsigned int y=0; y<box->height; y+=1) {
-    if (curRowLengths[box->top+y]) {
-      for (unsigned int x=0; x<box->width; x+=1) {
-        if (box->left+x < curRowLengths[box->top+y] - (curRows[box->top+y][curRowLengths[box->top+y]-1]=='\n')) {
-          buffer[y*box->width+x].text = curRows[box->top+y][box->left+x];
+    unsigned length = curRowLengths[box->top+y];
+    unsigned offset; /* Position in unicode codepoints */
+    unsigned sx; /* Position in screen offset */
+    unsigned right = box->left+box->width;
+
+    if (length && curRows[box->top+y][length-1] == WC_C('\n'))
+        /* Do not show trailing \n */
+        length--;
+
+    /* Process text of the line while peeking characters */
+    for (offset=0, sx=0; offset < length && sx<right; offset+=1) {
+      wchar_t wc = curRows[box->top+y][offset];
+
+      if (wc == '\t') {
+        /* Complete with spaces up to next tab */
+        unsigned nexttab = (sx+1+7)&~7;
+        for ( ; sx<nexttab; sx+=1) {
+          if (sx >= box->left && sx<right)
+            buffer[y*box->width+sx-box->left].text = WC_C(' ');
         }
+        continue;
+      }
+
+      /* XXX: Ignoring combining characters */
+      if (getCharacterWidth(wc) != 0) {
+        if (sx >= box->left && sx<right)
+          buffer[y*box->width+sx-box->left].text = wc;
+        sx+=1;
       }
     }
   }
@@ -1377,7 +1774,7 @@ static int
 AtSpi2GenerateKeyboardEvent (dbus_uint32_t keysym, enum key_type_e key_type)
 {
   DBusMessage *msg, *reply;
-  char *s = "";
+  const char *s = "";
 
   msg = new_method_call(SPI2_DBUS_INTERFACE_REG, SPI2_DBUS_PATH_DEC, SPI2_DBUS_INTERFACE_DEC, "GenerateKeyboardEvent");
   if (!msg)
@@ -1395,12 +1792,13 @@ insertKey_AtSpi2Screen (ScreenKey key) {
   long keysym;
   int modMeta=0, modControl=0;
 
+  mapScreenKey(&key);
   setScreenKeyModifiers(&key, SCR_KEY_CONTROL);
 
   if (isSpecialKey(key)) {
     switch (key & SCR_KEY_CHAR_MASK) {
 #ifdef HAVE_X11_KEYSYM_H
-      case SCR_KEY_ENTER:         keysym = XK_KP_Enter;  break;
+      case SCR_KEY_ENTER:         keysym = XK_Return;    break;
       case SCR_KEY_TAB:           keysym = XK_Tab;       break;
       case SCR_KEY_BACKSPACE:     keysym = XK_BackSpace; break;
       case SCR_KEY_ESCAPE:        keysym = XK_Escape;    break;
@@ -1516,6 +1914,69 @@ insertKey_AtSpi2Screen (ScreenKey key) {
   }
 }
 
+static int
+setSelection_AtSpi2Screen (int beginOffset, int endOffset) {
+  dbus_bool_t result;
+  DBusMessage *msg, *reply;
+  DBusMessageIter iter;
+  dbus_int32_t num = 0;
+  dbus_int32_t begin = beginOffset;
+  dbus_int32_t end = endOffset;
+
+  if (!curSender)
+    return 0;
+
+  msg = new_method_call(curSender, curPath, SPI2_DBUS_INTERFACE_TEXT, "SetSelection");
+  if (!msg)
+    return 0;
+  dbus_message_append_args(msg, DBUS_TYPE_INT32, &num, DBUS_TYPE_INT32, &begin, DBUS_TYPE_INT32, &end, DBUS_TYPE_INVALID);
+  reply = send_with_reply_and_block(bus, msg, 1000, "setting selection");
+  if (!reply)
+    return 0;
+
+  dbus_message_iter_init(reply, &iter);
+  if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_BOOLEAN) {
+    logMessage(LOG_CATEGORY(SCREEN_DRIVER),
+               "SetSelection didn't return a boolean but '%c'", dbus_message_iter_get_arg_type(&iter));
+    result = 0;
+    goto out;
+  }
+  dbus_message_iter_get_basic(&iter, &result);
+
+out:
+  dbus_message_unref(reply);
+  return result;
+}
+
+static int
+highlightRegion_AtSpi2Screen (int left, int right, int top, int bottom) {
+  int begin, end;
+
+  /* It is safe to play with selections only with terminals */
+  if (!isTerminal()) return 0;
+
+  if (top != bottom)
+    /* AtSpi selection only supports linear selection */
+    return 0;
+
+  begin = findCoordinates(left, top);
+  if (begin == -1)
+    return 0;
+  end = findCoordinates(right, bottom);
+  if (end == -1)
+    return 0;
+
+  return setSelection_AtSpi2Screen(begin, end+1);
+}
+
+static int
+unhighlightRegion_AtSpi2Screen (void) {
+  /* It is safe to play with selections only with terminals */
+  if (!isTerminal()) return 0;
+
+  return setSelection_AtSpi2Screen(0, 0);
+}
+
 static void
 scr_initialize (MainScreen *main) {
   initializeRealScreen(main);
@@ -1524,6 +1985,8 @@ scr_initialize (MainScreen *main) {
   main->base.describe = describe_AtSpi2Screen;
   main->base.readCharacters = readCharacters_AtSpi2Screen;
   main->base.insertKey = insertKey_AtSpi2Screen;
+  main->base.highlightRegion = highlightRegion_AtSpi2Screen;
+  main->base.unhighlightRegion = unhighlightRegion_AtSpi2Screen;
   main->base.currentVirtualTerminal = currentVirtualTerminal_AtSpi2Screen;
   main->processParameters = processParameters_AtSpi2Screen;
   main->construct = construct_AtSpi2Screen;
