@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2024 by The BRLTTY Developers.
+ * Copyright (C) 1995-2025 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -69,7 +69,7 @@ readCharacters_PcBiosScreen (const ScreenBox *box, ScreenCharacter *buffer) {
   for (row=box->top; row<box->top+box->height; ++row)
     for (col=box->left; col<box->left+box->width; ++col) {
       buffer->text = _farnspeekb(offset + row*description.cols*2 + col*2);
-      buffer->attributes = _farnspeekb(offset + row*description.cols*2 + col*2 + 1);
+      buffer->color.vgaAttributes = _farnspeekb(offset + row*description.cols*2 + col*2 + 1);
       buffer++;
     }
   return 1;
@@ -142,7 +142,7 @@ insertMapped (ScreenKey key) {
       case SCR_KEY_FUNCTION + 12:
         return(simulateKey(88, 0));
       default:
-        logMessage(LOG_WARNING, "Key %4.4X not suported.", key);
+        logMessage(LOG_WARNING, "Key %4.4X not supported.", key);
         return 0;
     }
     end = sequence + strlen(sequence);
