@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2024 by The BRLTTY Developers.
+ * Copyright (C) 1995-2025 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -295,14 +295,14 @@ readCharacters_WindowsScreen (const ScreenBox *box, ScreenCharacter *buffer) {
 
     for (x=0, xx=0; x<box->width && xx < read; x++, xx++) {
       int i;
-      buffer[y*box->width+x].attributes = bufAttr[xx];
+      buffer[y*box->width+x].color.vgaAttributes = bufAttr[xx];
       for (i = 1; i < getCharacterWidth(buffer[y*box->width+x].text); i++) {
 	x++;
-        buffer[y*box->width+x].attributes = bufAttr[xx];
+        buffer[y*box->width+x].color.vgaAttributes = bufAttr[xx];
       }
     }
     for ( ; x < box->width; x++)
-      buffer[y*box->width+x].attributes = 0;
+      buffer[y*box->width+x].color.vgaAttributes = 0;
   }
 
   free(buf);
@@ -438,7 +438,7 @@ insertKey_WindowsScreen (ScreenKey key) {
       case SCR_KEY_FUNCTION + 21: vk = VK_F22;    break;
       case SCR_KEY_FUNCTION + 22: vk = VK_F23;    break;
       case SCR_KEY_FUNCTION + 23: vk = VK_F24;    break;
-      default: logMessage(LOG_WARNING, "Key %4.4X not suported.", key);
+      default: logMessage(LOG_WARNING, "Key %4.4X not supported.", key);
                return 0;
     }
   } else {
