@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2024 by The BRLTTY Developers.
+ * Copyright (C) 1995-2025 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -480,7 +480,7 @@ spk_destruct (SpeechSynthesizer *spk) {
   closeSoundDevice();
 
   if (speechQueue) {
-    deallocateQueue(speechQueue);
+    destroyQueue(speechQueue);
     speechQueue = NULL;
   }
 
