@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2024 by The BRLTTY Developers.
+ * Copyright (C) 1995-2025 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -40,18 +40,32 @@ extern int usbNextDescriptor (
   UsbDevice *device,
   const UsbDescriptor **descriptor
 );
+
 extern const UsbConfigurationDescriptor *usbConfigurationDescriptor (
   UsbDevice *device
 );
+
+extern const UsbInterfaceAssociationDescriptor *usbInterfaceAssociationDescriptor (
+  UsbDevice *device,
+  unsigned char interface
+);
+
+extern int usbIsAssociatedInterface (
+  const UsbInterfaceAssociationDescriptor *iad,
+  unsigned char interface
+);
+
 extern const UsbInterfaceDescriptor *usbInterfaceDescriptor (
   UsbDevice *device,
   unsigned char interface,
   unsigned char alternative
 );
+
 extern unsigned int usbAlternativeCount (
   UsbDevice *device,
   unsigned char interface
 );
+
 extern const UsbEndpointDescriptor *usbEndpointDescriptor (
   UsbDevice *device,
   unsigned char endpointAddress
@@ -229,11 +243,6 @@ extern int usbAddInputFilter (UsbDevice *device, UsbInputFilter *filter);
 extern const UsbSerialOperations *usbGetSerialOperations (UsbDevice *device);
 extern int usbSetSerialParameters (UsbDevice *device, const SerialParameters *parameters);
 
-typedef struct {
-  const UsbChannelDefinition *definition;
-  UsbDevice *device;
-} UsbChannel;
-
 extern UsbChannel *usbOpenChannel (const UsbChannelDefinition *definitions, const char *identifier);
 extern void usbCloseChannel (UsbChannel *channel);
 extern const char *usbMakeChannelIdentifier (UsbChannel *channel, char *buffer, size_t size);
