@@ -116,7 +116,7 @@ destructBrailleDisplay (BrailleDisplay *brl) {
   }
 
   if (brl->acknowledgements.messages) {
-    deallocateQueue(brl->acknowledgements.messages);
+    destroyQueue(brl->acknowledgements.messages);
     brl->acknowledgements.messages = NULL;
   }
 
