@@ -20,7 +20,9 @@
 /* api_client.c handles connection with BrlApi */
 
 #ifdef __ANDROID__
+#ifndef __ANDROID_API__
 #define __ANDROID_API__ 21
+#endif /* __ANDROID_API__ */
 #endif /* __ANDROID__ */
 
 #define WIN_ERRNO_STORAGE_CLASS static
@@ -1804,7 +1806,7 @@ int BRLAPI_STDCALL brlapi__enterTtyModeWithPath(brlapi_handle_t *handle, const i
 	    }
 	  }
 
-	  if (chosen >= 0) sd_session_get_vt(sessions[i], &vtnr);
+	  if (chosen >= 0) sd_session_get_vt(sessions[chosen], &vtnr);
 
 	  for (i = 0; i < nsessions; i++) free(sessions[i]);
 	  free(sessions);
