@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2024 by The BRLTTY Developers.
+ * Copyright (C) 1995-2025 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -25,8 +25,8 @@
 
 #include "log.h"
 #include "strfmt.h"
-#include "program.h"
 #include "cmdline.h"
+#include "options.h"
 #include "file.h"
 #include "get_select.h"
 #include "brl_dots.h"
@@ -134,14 +134,14 @@ showUndefinedCharacters (TextTableData *ttd) {
 static char *opt_charset;
 static char *opt_inputFormat;
 static char *opt_outputFormat;
-static char *opt_tablesDirectory;
+char *opt_tablesDirectory;
 static int opt_edit;
 
 #ifdef HAVE_UNDEFINED_CHARACTERS_SUPPORT
 static int opt_undefined;
 #endif /* HAVE_UNDEFINED_CHARACTERS_SUPPORT */
 
-BEGIN_OPTION_TABLE(programOptions)
+BEGIN_COMMAND_LINE_OPTIONS(programOptions)
   { .word = "edit",
     .letter = 'e',
     .setting.flag = &opt_edit,
@@ -182,10 +182,38 @@ BEGIN_OPTION_TABLE(programOptions)
     .argument = "directory",
     .setting.string = &opt_tablesDirectory,
     .internal.setting = TABLES_DIRECTORY,
-    .internal.adjust = fixInstallPath,
+    .internal.adjust = toAbsoluteInstallPath,
     .description = strtext("Path to directory containing text tables.")
   },
-END_OPTION_TABLE(programOptions)
+END_COMMAND_LINE_OPTIONS(programOptions)
+
+static const char *inputPath;
+static const char *outputPath;
+
+BEGIN_COMMAND_LINE_PARAMETERS(programParameters)
+  { .name = "input",
+    .description = "the name of (or path to) the input table",
+    .setting = &inputPath,
+  },
+
+  { .name = "output",
+    .description = "the name of (or path to) the output table",
+    .setting = &outputPath,
+    .optional = 1,
+  },
+END_COMMAND_LINE_PARAMETERS(programParameters)
+
+BEGIN_COMMAND_LINE_NOTES(programNotes)
+END_COMMAND_LINE_NOTES
+
+BEGIN_COMMAND_LINE_DESCRIPTOR(programDescriptor)
+  .name = "brltty-ttb",
+  .purpose = strtext("Check/edit a text (computer braille) table, or convert it from one format to another."),
+
+  .options = &programOptions,
+  .parameters = &programParameters,
+  .notes = COMMAND_LINE_NOTES(programNotes),
+END_COMMAND_LINE_DESCRIPTOR
 
 static const BrlDotTable dotsInternal = {
   BRL_DOT_1, BRL_DOT_2, BRL_DOT_3, BRL_DOT_4,
@@ -304,7 +332,7 @@ writeHeaderComment (FILE *file, CommentWriter *writeComment) {
   char buffer[0X80];
 
   STR_BEGIN(buffer, sizeof(buffer));
-  STR_PRINTF("generated by %s", programName);
+  STR_PRINTF("generated by %s", getProgramName());
   if (*opt_charset) STR_PRINTF(": charset=%s", opt_charset);
   STR_END;
 
@@ -1187,8 +1215,6 @@ getFormatEntry (const char *name, const char *path, const char *description) {
   exit(PROG_EXIT_SYNTAX);
 }
 
-static const char *inputPath;
-static const char *outputPath;
 static const FormatEntry *inputFormat;
 static const FormatEntry *outputFormat;
 
@@ -1292,10 +1318,11 @@ convertTable (void) {
 
 #include "get_curses.h"
 #ifndef GOT_CURSES
-#define refresh() fflush(stdout)
-#define printw printf
-#define erase() printf("\r\n\v")
-#define beep() printf("\a")
+#include "cmdput.h"
+#define refresh() putFlush()
+#define printw putf
+#define erase() putString("\r\n\v")
+#define beep() putString("\a")
 
 static int inputAttributesChanged;
 
@@ -1344,7 +1371,7 @@ typedef struct {
 #ifdef ENABLE_API
 static int
 haveBrailleDisplay (EditTableData *etd) {
-  return etd->brlapiFileDescriptor != (brlapi_fileDescriptor)-1;
+  return etd->brlapiFileDescriptor != BRLAPI_INVALID_FILE_DESCRIPTOR;
 }
 
 static void
@@ -1359,7 +1386,7 @@ setBrlapiError (EditTableData *etd, const char *function) {
 static void
 releaseBrailleDisplay (EditTableData *etd) {
   brlapi_closeConnection();
-  etd->brlapiFileDescriptor = (brlapi_fileDescriptor)-1;
+  etd->brlapiFileDescriptor = BRLAPI_INVALID_FILE_DESCRIPTOR;
 }
 
 static int
@@ -2502,52 +2529,27 @@ editTable (void) {
 
 int
 main (int argc, char *argv[]) {
+  PROCESS_COMMAND_LINE(programDescriptor, argc, argv);
+
   ProgramExitStatus exitStatus;
-
-  {
-    const CommandLineDescriptor descriptor = {
-      .options = &programOptions,
-      .applicationName = "brltty-ttb",
-
-      .usage = {
-        .purpose = strtext("Check/edit a text (computer braille) table, or convert it from one format to another."),
-        .parameters = "input-table [output-table]",
-      }
-    };
-
-    PROCESS_OPTIONS(descriptor, argc, argv);
-  }
-
-  if (argc == 0) {
-    logMessage(LOG_ERR, "missing input table");
-    return PROG_EXIT_SYNTAX;
-  }
-  inputPath = *argv++, argc--;
-
-  if (argc > 0) {
-    outputPath = *argv++, argc--;
-  } else if (opt_outputFormat && *opt_outputFormat) {
-    const char *extension = locatePathExtension(inputPath);
-    int prefix = extension? (extension - inputPath): strlen(inputPath);
-    char buffer[prefix + 1 + strlen(opt_outputFormat) + 1];
-
-    snprintf(buffer, sizeof(buffer), "%.*s.%s", prefix, inputPath, opt_outputFormat);
-
-    if (!(outputPath = strdup(buffer))) {
-      logMallocError();
-      return PROG_EXIT_FATAL;
-    }
-  } else {
-    outputPath = NULL;
-  }
-
-  if (argc > 0) {
-    logMessage(LOG_ERR, "too many parameters");
-    return PROG_EXIT_SYNTAX;
-  }
-
   inputFormat = getFormatEntry(opt_inputFormat, inputPath, "input");
 
+  if (!outputPath) {
+    if (opt_outputFormat && *opt_outputFormat) {
+      const char *extension = locatePathExtension(inputPath);
+      int prefix = extension? (extension - inputPath): strlen(inputPath);
+      char buffer[prefix + 1 + strlen(opt_outputFormat) + 1];
+      snprintf(buffer, sizeof(buffer), "%.*s.%s", prefix, inputPath, opt_outputFormat);
+
+      if (!(outputPath = strdup(buffer))) {
+        logMallocError();
+        return PROG_EXIT_FATAL;
+      }
+    } else {
+      outputPath = NULL;
+    }
+  }
+
   if (outputPath) {
     outputFormat = getFormatEntry(opt_outputFormat, outputPath, "output");
   } else {
