@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2024 by The BRLTTY Developers.
+ * Copyright (C) 1995-2025 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -63,7 +63,7 @@ toDifferentLine (
     ScreenCharacter characters1[width];
     unsigned int skipped = 0;
 
-    if ((isSameCharacter == isSameText) && ses->displayMode) isSameCharacter = isSameAttributes;
+    if ((isSameCharacter == isSameText) && ses->displayMode) isSameCharacter = isSameScreenColor;
     readScreen(from, ses->winy, width, 1, characters1);
 
     do {
@@ -169,7 +169,7 @@ testIndent (int column, int row, void *data UNUSED) {
 
   while (column >= 0) {
     wchar_t text = characters[column].text;
-    if (text != WC_C(' ')) return 1;
+    if (!iswspace(text)) return 1;
     column -= 1;
   }
 
@@ -264,8 +264,7 @@ toPreviousNonblankWindow (void) {
 
     for (charIndex=charCount-1; charIndex>=0; charIndex-=1) {
       wchar_t text = characters[charIndex].text;
-
-      if (text != WC_C(' ')) break;
+      if (!iswspace(text)) break;
     }
 
     if (showScreenCursor() &&
@@ -310,8 +309,7 @@ toNextNonblankWindow (void) {
 
     for (charIndex=0; charIndex<charCount; charIndex+=1) {
       wchar_t text = characters[charIndex].text;
-
-      if (text != WC_C(' ')) break;
+      if (!iswspace(text)) break;
     }
 
     if (showScreenCursor() &&
@@ -374,10 +372,10 @@ handleNavigationCommands (int command, void *data) {
       break;
 
     case BRL_CMD_ATTRUP:
-      upDifferentLine(isSameAttributes);
+      upDifferentLine(isSameScreenColor);
       break;
     case BRL_CMD_ATTRDN:
-      downDifferentLine(isSameAttributes);
+      downDifferentLine(isSameScreenColor);
       break;
 
     case BRL_CMD_PRPGRPH: {
@@ -484,7 +482,7 @@ handleNavigationCommands (int command, void *data) {
           int column = 0;
 
           while (column < length) {
-            if (characters[column].text == WC_C(' ')) break;
+            if (iswspace(characters[column].text)) break;
             column += 1;
           }
 
@@ -577,8 +575,7 @@ handleNavigationCommands (int command, void *data) {
 
               for (charIndex=0; charIndex<charCount; charIndex+=1) {
                 wchar_t text = characters[charIndex].text;
-
-                if (text != WC_C(' ')) break;
+                if (!iswspace(text)) break;
               }
 
               if (charIndex == charCount) goto wrapUp;
@@ -651,8 +648,7 @@ handleNavigationCommands (int command, void *data) {
 
               for (charIndex=0; charIndex<charCount; charIndex+=1) {
                 wchar_t text = characters[charIndex].text;
-
-                if (text != WC_C(' ')) break;
+                if (!iswspace(text)) break;
               }
 
               if (charIndex == charCount) goto wrapDown;
