@@ -224,23 +224,6 @@ anchorRelativePath (char **path, const char *anchor) {
   return ok;
 }
 
-char *opt_helpersDirectory = HELPERS_DIRECTORY;
-
-char *
-makeHelperPath (const char *name) {
-  char *path = NULL;
-  char *directory = NULL;
-
-  if (changeStringSetting(&directory, opt_helpersDirectory)) {
-    if (toAbsoluteInstallPath(&directory)) {
-      path = makePath(directory, name);
-    }
-  }
-
-  if (directory) free(directory);
-  return path;
-}
-
 int
 hasFileExtension (const char *path, const char *extension) {
   const char *tail = locatePathExtension(path);
@@ -557,6 +540,32 @@ makeWritablePath (const char *file) {
   return makeDirectoryPath(&opt_writableDirectory, file);
 }
 
+const char *
+getDevicesDirectory (void) {
+  static const char *devicesDirectory = NULL;
+
+  if (!devicesDirectory) {
+    const char *directory = makeWritablePath("dev");
+
+    if (directory) {
+      if (ensureDirectory(directory, 0)) {
+        devicesDirectory = directory;
+      }
+    }
+
+    if (devicesDirectory) logMessage(LOG_DEBUG, "devices directory: %s", devicesDirectory);
+  }
+
+  return devicesDirectory;
+}
+
+char *
+makeDevicesPath (const char *file) {
+  const char *directory = getDevicesDirectory();
+  if (directory) return makePath(directory, file);
+  return NULL;
+}
+
 char *
 getWorkingDirectory (void) {
 #if defined(GRUB_RUNTIME)
@@ -1299,7 +1308,7 @@ createAnonymousPipe (FileDescriptor *pipeInput, FileDescriptor *pipeOutput) {
 }
 #endif /* basic file/socket descriptor operations */
 
-void
+int
 writeWithConsoleEncoding (FILE *stream, const char *bytes, size_t count) {
   static const char *consoleEncoding = NULL;
   if (!consoleEncoding) consoleEncoding = getConsoleEncoding();
@@ -1341,18 +1350,15 @@ writeWithConsoleEncoding (FILE *stream, const char *bytes, size_t count) {
       &outputNext, &outputLeft
     );
 
-    if (result != -1) {
-      size_t length = outputNext - outputBuffer;
-      outputBuffer[length] = 0;
-      fputs(outputBuffer, stream);
-    }
-
-    return;
+    if (result == -1) return 0;
+    fwrite(outputBuffer, 1, (outputNext - outputBuffer), stream);
+    return !ferror(stream);
   }
 ENCODING_NOT_SUPPORTED:
 #endif /* HAVE_ICONV_H */
 
   fwrite(bytes, 1, count, stream);
+  return !ferror(stream);
 }
 
 #ifdef GOT_SOCKETS
