@@ -29,6 +29,7 @@ static const CommandListEntry commandList_modes[] = {
   { .code = BRL_CMD_DISPMD },
   { .code = BRL_CMD_FREEZE },
   { .code = BRL_CMD_BLK(DESCCHAR) },
+  { .code = BRL_CMD_BLK(COLOR) },
   { .code = BRL_CMD_TIME },
   { .code = BRL_CMD_INDICATORS },
   { .code = BRL_CMD_BLK(CONTEXT) },
@@ -201,11 +202,16 @@ static const CommandListEntry commandList_speak[] = {
   { .code = BRL_CMD_SPEAK_NEXT_CHAR },
   { .code = BRL_CMD_SPEAK_FRST_CHAR },
   { .code = BRL_CMD_SPEAK_LAST_CHAR },
+  { .code = BRL_CMD_SPEAK_CURR_PWRD },
+  { .code = BRL_CMD_SPELL_CURR_PWRD },
+  { .code = BRL_CMD_SPEAK_PREV_PWRD },
+  { .code = BRL_CMD_SPEAK_NEXT_PWRD },
   { .code = BRL_CMD_SPEAK_CURR_WORD },
   { .code = BRL_CMD_SPELL_CURR_WORD },
   { .code = BRL_CMD_SPEAK_PREV_WORD },
   { .code = BRL_CMD_SPEAK_NEXT_WORD },
   { .code = BRL_CMD_SPEAK_CURR_LINE },
+  { .code = BRL_CMD_SPELL_CURR_LINE },
   { .code = BRL_CMD_SPEAK_PREV_LINE },
   { .code = BRL_CMD_SPEAK_NEXT_LINE },
   { .code = BRL_CMD_SPEAK_FRST_LINE },
