@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2024 by The BRLTTY Developers.
+ * Copyright (C) 1995-2025 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -19,6 +19,7 @@
 #ifndef BRLTTY_INCLUDED_SCR
 #define BRLTTY_INCLUDED_SCR
 
+#include "strfmt_types.h"
 #include "scr_types.h"
 #include "ktb_types.h"
 #include "driver.h"
@@ -27,6 +28,12 @@
 extern "C" {
 #endif /* __cplusplus */
 
+extern int sameScreenColors (const ScreenColor *color1, const ScreenColor *color2);
+extern int sameBackgroundColors (const ScreenColor *color1, const ScreenColor *color2);
+
+extern unsigned char getScreenColorAttributes (const ScreenColor *color);
+extern STR_DECLARE_FORMATTER(formatScreenColor, const ScreenColor *color);
+
 extern int isMainScreen (void);
 
 /* Routines which apply to the current screen. */
@@ -35,7 +42,9 @@ extern int refreshScreen (void);
 extern void describeScreen (ScreenDescription *);		/* get screen status */
 extern int readScreen (short left, short top, short width, short height, ScreenCharacter *buffer);
 extern int readScreenText (short left, short top, short width, short height, wchar_t *buffer);
+
 extern int insertScreenKey (ScreenKey key);
+extern ScreenPasteMode getScreenPasteMode (void);
 extern int routeScreenCursor (int column, int row, int screen);
 extern int highlightScreenRegion (int left, int right, int top, int bottom);
 extern int unhighlightScreenRegion (void);
@@ -61,13 +70,10 @@ readScreenRow (int row, int width, ScreenCharacter *buffer) {
   return readScreenRows(row, width, 1, buffer);
 }
 
-/* Routines which apply to the routing screen.
- * An extra `thread' for the cursor routing subprocess.
- * This is needed because the forked subprocess shares its parent's
- * file descriptors.  A readScreen equivalent is not needed.
- */
-extern int constructRoutingScreen (void);
-extern void destructRoutingScreen (void);
+static inline int
+readScreenCharacter (int column, int row, ScreenCharacter *character) {
+  return readScreen(column, row, 1, 1, character);
+}
 
 extern const ScreenDriver *screen;
 extern const ScreenDriver noScreen;
