@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2024 by The BRLTTY Developers.
+ * Copyright (C) 1995-2025 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -230,12 +230,21 @@ usbSetFlowControl_CDC_ACM (UsbDevice *device, SerialFlowControl flow) {
 
 static const UsbInterfaceDescriptor *
 usbFindCommunicationInterface (UsbDevice *device) {
+  const UsbInterfaceAssociationDescriptor *iad = usbInterfaceAssociationDescriptor(device, device->interface->bInterfaceNumber);
   const UsbDescriptor *descriptor = NULL;
 
   while (usbNextDescriptor(device, &descriptor)) {
     if (descriptor->header.bDescriptorType == UsbDescriptorType_Interface) {
-      if (descriptor->interface.bInterfaceClass == 0X02) {
-        return &descriptor->interface;
+      const UsbInterfaceDescriptor *interface = &descriptor->interface;
+
+      if (interface->bInterfaceClass == 0X02) {
+        if (iad) {
+          if (!usbIsAssociatedInterface(iad, interface->bInterfaceNumber)) {
+            continue;
+          }
+        }
+
+        return interface;
       }
     }
   }
