@@ -461,7 +461,7 @@ usbSubmitRequest (
           break;
 
         default:
-          logMessage(LOG_ERR, "USB endpoint direction not suppported: %d",
+          logMessage(LOG_ERR, "USB endpoint direction not supported: %d",
                      eptx->transferDirection);
           errno = ENOSYS;
           break;
@@ -704,7 +704,7 @@ usbAllocateEndpointExtension (UsbEndpoint *endpoint) {
       logMessage(LOG_ERR, "USB pipe not found: ept=%02X",
                  endpoint->descriptor->bEndpointAddress);
 
-      deallocateQueue(eptx->completedRequests);
+      destroyQueue(eptx->completedRequests);
     } else {
       logSystemError("USB completed request queue allocate");
     }
@@ -720,7 +720,7 @@ usbAllocateEndpointExtension (UsbEndpoint *endpoint) {
 void
 usbDeallocateEndpointExtension (UsbEndpointExtension *eptx) {
   if (eptx->completedRequests) {
-    deallocateQueue(eptx->completedRequests);
+    destroyQueue(eptx->completedRequests);
     eptx->completedRequests = NULL;
   }
 
