@@ -22,7 +22,7 @@ listInstalledPackages() {
 }
 
 installPackages() {
-   pacman --sync --quiet --noprogressbar --noconfirm -- "${@}"
+   pacman --sync --needed --quiet --noprogressbar --noconfirm -- "${@}"
 }
 
 removePackages() {
